# pylint: disable=missing-module-docstring, wrong-import-position, import-error, too-few-public-methods too-many-arguments
import sys
import os

currentPath = os.path.dirname(os.path.realpath(__file__))
parentPath = os.path.dirname(currentPath)
sys.path.append(parentPath)

from bad.encoding.hands import Hands

class PrivateFeatures:
    '''private features'''
    def __init__(self, observation: dict) -> None:
        '''init'''
        self.observation = observation
        self.hands = self.convert_hands()

    def convert_hands(self) -> Hands:
        '''returns hands'''
        return Hands(self.observation)



# pylint: disable=missing-module-docstring, wrong-import-position, import-error too-few-public-methods too-many-arguments
import sys
import os
import numpy as np

currentPath = os.path.dirname(os.path.realpath(__file__))
parentPath = os.path.dirname(currentPath)
sys.path.append(parentPath)

from bad.encoding.publicfeatures import PublicFeatures
from bad.encoding.privatefeatures import PrivateFeatures

class Observation:
    '''observation'''
    def __init__(self, observation: dict):
        self.public_features = PublicFeatures(observation)
        self.private_features = PrivateFeatures(observation)

    def to_array(self) -> list:
        '''one single vecor'''
        result = np.concatenate(( \
            self.public_features.life_tokens_left, \
            self.public_features.hint_tokens_left, \
            self.public_features.firework.red, \
            self.public_features.firework.yellow, \
            self.public_features.firework.green, \
            self.public_features.firework.white, \
            self.public_features.firework.blue, \
            self.public_features.last_action, \
            self.public_features.legal_actions.vector, \
            self.public_features.current_player, \
            self.private_features.hands.own_cards,
            self.private_features.hands.other_cards
            ))
        return result



# pylint: disable=missing-module-docstring too-few-public-methods)
import numpy as np


class BayesianAction:
    '''Bayesian Action'''
    def __init__(self, actions: np.ndarray) -> None:
        self.actions = actions

    def decode_action(self, legal_moves:np.ndarray) -> int:
        '''returns a choice'''
        legal_moves_int = legal_moves.tolist()
        all_actions = self.actions.copy()

        for action in range(0, all_actions.size):
            exists: bool = legal_moves_int.count(action) > 0
            if not exists:
                all_actions[action] = -float('inf')

        result = np.argmax(all_actions, axis=0)
        return int(result)



# pylint: disable=missing-module-docstring, wrong-import-position, import-error
import sys
import os
import tensorflow as tf

currentPath = os.path.dirname(os.path.realpath(__file__))
parentPath = os.path.dirname(currentPath)
sys.path.append(parentPath)

from bad.encoding.observation import Observation
from bad.bayesianaction import BayesianAction

class ActionNetwork():
    ''' action network '''

    def __init__(self) -> None:
        self.model = None

    def build(self, observation: Observation, max_action: int) -> None:
        '''build'''
        shape = observation.to_array().shape
        self.model = tf.keras.Sequential([
            tf.keras.Input(shape=shape, name="input"),
            tf.keras.layers.Dense(384, activation="relu", name="layer1"),
            tf.keras.layers.Dense(384, activation="relu", name="layer2"),
            tf.keras.layers.Dense(max_action, activation='softmax', name='Output_Layer')
        ])

    def print_summary(self):
        '''print summary'''
        self.model.summary()

    def get_action(self, observation: Observation) -> BayesianAction:
        '''get action'''
        network_input = observation.to_array()
        reshaped = tf.reshape(network_input, [1, network_input.shape[0]])

        result = self.model(reshaped)
        return BayesianAction(result.numpy()[0])
    def train_observation(self, observation: Observation) -> BayesianAction:
        '''input observation, output action'''
        return self.get_action(observation)



# pylint: disable=missing-module-docstring, wrong-import-position, import-error, no-member, no-name-in-module, unnecessary-pass, too-few-public-methods

class BadAgentActingResult:
    ''''bad agent acting result'''
    def __init__(self, observation_after_step: dict, done: bool, reward:int) -> None:
        self.observation_after_step = observation_after_step
        self.done = done
        self.reward = reward



# pylint: disable=missing-module-docstring, wrong-import-position, import-error too-few-public-methods too-many-arguments

import string
import numpy as np
import tensorflow as tf


class ColorToIntConverter:
    '''color to int converter'''

    def __init__(self) -> None:
        '''init'''
        self.colors = {'None': 0, 'R': 1, 'Y': 2, 'G': 3, 'W': 4, 'B': 5}

    def convert(self, col: string) -> np.ndarray:
        '''convert'''
        return tf.keras.utils.to_categorical(self.convert_color_to_int(col), num_classes=6, \
            dtype=int)

    def convert_color_to_int(self, col: string) -> int:
        '''convert color to int'''
        return self.colors[col]



# pylint: disable=missing-module-docstring, wrong-import-position, import-error, too-few-public-methods too-many-arguments
import numpy as np
import tensorflow as tf

class LegalActions:
    '''legal moves'''
    def __init__(self, observation: dict) -> None:
        legal_actions:list = observation['legal_actions']
        max_action:int = observation ['max_action']
        self.vector = np.empty(0, int)
        for one_action in range(max_action):
            legal_actions_list = legal_actions.tolist()
            exists: bool = legal_actions_list.count(one_action) > 0
            self.vector = np.append(self.vector, self.get_encoded_action_exists(exists))

    def get_encoded_action_exists(self, exists: bool) -> np.ndarray:
        '''get encoded action exists'''
        if exists:
            return tf.keras.utils.to_categorical(1, num_classes=2, dtype=int)

        return tf.keras.utils.to_categorical(0, num_classes=2, dtype=int)



# pylint: disable=missing-module-docstring
from setuptools import find_packages
from skbuild import setup

setup(
    name='ws2223-group7-hanabi-learning-environment_bad',
    version='0.0.6',
    description='ws2223-group7 Learning environment for the game of hanabi bad agent.',
    long_description='ws2223-group7 Learning environment for the game of hanabi bad agent.',
    long_description_content_type="text/markdown",
    author='ws2223-group7/hanabi-learning-environment_bad',
    packages=find_packages(),
    install_requires=['cffi'],
    python_requires=">=3.6",
    classifiers = [
    "Programming Language :: Python :: 3",
    "Operating System :: POSIX :: Linux",
    ],
    entry_points={
    'console_scripts': [
        'ws2223-group7-hanabi-learning-environment-bad = bad.main:main',
    ],
    }
)
